import pygame
from pygame.locals import *
from sys import exit
from patterns import *
import json
import urllib2

grass = pygame.image.load("grass1.png")
house = pygame.image.load("house.png")
shop = pygame.image.load("shop.png")
farms = pygame.image.load("farm.png")
bigfarms = pygame.image.load("bigfarm.png")
supermarket = pygame.image.load("super.png")

welcome = pygame.image.load("welcome.png")
intro = pygame.image.load("intro.png")
run = pygame.image.load("run.png")

won = pygame.image.load("won.png")
lost = pygame.image.load("lost.png")

CELL_WIDTH = 44
CELL_HEIGHT = 44

UNBUILT = 0
COMMERCIAL = 1
RESIDENTIAL = 2
FARM = 3
BIGFARM = 5
SUPERMARKET = 6

class Cell (object): 
    def __init__ (self, r, c):
        self.r = r
        self.c = c
        x = c*CELL_WIDTH
        y = r*CELL_HEIGHT
        self.rect = Rect((x, y), (CELL_WIDTH, CELL_HEIGHT))
        self.color = (255, 255, 0)
        self.type_next = None
        self.type = UNBUILT
                
    def draw (self, screen):
        if (self.type_next != None):
            self.type = self.type_next
            self.type_next = None

        if self.type == UNBUILT:
            screen.blit(grass, self.rect)

        if self.type == COMMERCIAL:
            screen.blit(shop, self.rect)
        elif self.type == RESIDENTIAL:
            screen.blit(house, self.rect)
        elif self.type == FARM:
            screen.blit(farms, self.rect)
        elif self.type == BIGFARM:
            screen.blit(bigfarms, self.rect)
        elif self.type == SUPERMARKET:
            screen.blit(supermarket, self.rect)

    def check (self):
        count()
        HOUSES = counts['cres']
        SHOPS = counts['ccom']
        FARMS = counts['cfarm']
        BIGFARMS = counts['cbig']
        SUPERMARKETS = counts['super']
        (cc, cf, cr, cbf) = (0, 0, 0, 0)
        for dr in (-1, 0, 1):
            for dc in (-1, 0, 1):
                if (dr == 0 and dc == 0):
                    continue
                (c, r) = (self.c+dc, self.r+dr)
                if c>-1 and c<ncols and r>-1 and r<nrows:
                    if cells[r][c].type == RESIDENTIAL:
                        cr += 1
                    if cells[r][c].type == COMMERCIAL:
                        cc += 1
                    if cells[r][c].type == BIGFARM:
                        cbf += 1
                    elif cells[r][c].type == FARM:
                        cf += 1
# rule one: commercial zones will emerge to supply residential areas with groceries
        if self.type == UNBUILT and cr >= 3 and cc <= 0:
            self.type_next = COMMERCIAL

# rule two: the city is expanding on the account of surrounding farms:
        if (self.type == FARM or self.type == BIGFARM) and (cr > 0 or cc > 0):
            self.type_next = UNBUILT

# rule three: four farms in a square formation make a big farm:
        if self.type == UNBUILT and cf >= 3 and cbf < 1:
            self.type_next = BIGFARM    
        if self.type == BIGFARM and cf+cbf < 3:
            self.type_next = FARM

# rule four: shops need farms:
        if self.type == COMMERCIAL and FARMS+BIGFARMS < 1:
            print "There are no farms to supply grocery shops with food"
            self.type_next = RESIDENTIAL

# rule four: SUPERMARKETS need big farms
        if self.type == SUPERMARKET and BIGFARMS < 1:
            print "There are no industrial farms to supply supermarkets with uniformly looking food, the supermarkets will revert to small grocery shops."
            self.type_next = COMMERCIAL

counts = {'cres':0, 'ccom':0, 'cfarm':0, 'cbig':0, 'super':0}

#------------------game functions-----------------------#
## EXTERNAL

# read in the answers from django; returns (PRO_S, PRO_O) coupple of values
def openURL (url):
    """
    returns (page, actualurl)
    sets user_agent and resolves possible redirection
    realurl maybe different than url in the case of a redirect
    """    
    request = urllib2.Request(url)
    user_agent = "Mozilla/5.0 (X11; U; Linux x86_64; fr; rv:1.9.1.5) Gecko/20091109 Ubuntu/9.10 (karmic) Firefox/3.5.5"
    request.add_header("User-Agent", user_agent)
    pagefile=urllib2.urlopen(request)
    realurl = pagefile.geturl()
    return (pagefile, realurl)

def read_db():
    #(f, url) = openURL("http://localhost:8000/feedback/answers")
    (f, url) = openURL("http://how-do-you-like-them-apples.info/miss/goodfood/feedback/answers")
    data = f.read()
    #print data
    dict = json.loads(data)
    PRO_S = 0
    PRO_O = 0
    for item in dict:
        #print dict[item].values()
        PRO_S += dict[item].values()[0]
        PRO_O += dict[item].values()[1]
    return (PRO_S, PRO_O)

#------------------game functions-----------------------#
## INTERNAL

# check out the whole area, count all zones, returns a dictionary:
def count():
    global counts
    counts = {'cres':0, 'ccom':0, 'cfarm':0, 'cbig':0, 'super':0}
    nrows = len(cells)
    ncols = len(cells[0])
    for c in range(ncols):
        for r in range(nrows):
            if c>-1 and c<ncols and r>-1 and r<nrows:
                if cells[r][c].type == RESIDENTIAL:
                    counts['cres'] += 1
                elif cells[r][c].type == COMMERCIAL:
                    counts['ccom'] += 1
                elif cells[r][c].type == FARM:
                    counts['cfarm'] += 1
                elif cells[r][c].type == BIGFARM:
                    counts['cbig'] += 1
                elif cells[r][c].type == SUPERMARKET:
                    counts['super'] += 1
    return counts

### main loop funcions

#### USER INTERACTION ####
def which_key():
    global running, drawing, draw_type
    for event in pygame.event.get():
        if event.type == QUIT:
            exit()
        elif event.type == KEYDOWN and event.key == K_ESCAPE:
            exit()
        elif event.type == MOUSEBUTTONDOWN:
            drawing = True
        elif event.type == MOUSEBUTTONUP:
            drawing = False
        
        elif drawing:
            mrow = event.pos[1] / CELL_HEIGHT
            mcol = event.pos[0] / CELL_WIDTH
            c = cells[mrow][mcol]
            if draw_type == SUPERMARKET:
                if c.type == COMMERCIAL:
                    c.type_next = draw_type
            else:
                c.type_next = draw_type
            pygame.display.flip()
             
        elif event.type == KEYDOWN:
            if event.key == K_SPACE:
                running = not running
            
            elif event.key == K_r:
                (NEW_RES, NEW_COMM, NEW_FARM, NEW_BIGFARM) = calculation()
                    
            elif event.key == K_s:
                draw_type = SUPERMARKET
            
            elif event.key == K_g:
                draw_type = COMMERCIAL
            elif event.key == K_h:
                draw_type = RESIDENTIAL
            elif event.key == K_f:
                draw_type = FARM
            elif event.key == K_u:
                draw_type = UNBUILT
        
    return running, drawing, draw_type

#### RUN THE GAME ####
def run_game():
    global ncells
    for c in range(ncols):
        for r in range(nrows):
            cells[r][c].check()
    count()
    HOUSES = counts['cres']
    SHOPS = counts['ccom']
    FARMS = counts['cfarm']
    BIGFARMS = counts['cbig']
    SUPERMARKETS = counts['super']
    print "HOUSES:", HOUSES, "; SHOPS:", SHOPS, "; FARMS:", FARMS, "; BIGFARMS:", BIGFARMS, "; SUPERMARKETS:", SUPERMARKETS, ";"
    (NEW_RES, NEW_COMM, NEW_FARM, NEW_BIGFARM) = calculation()
    if BIGFARMS < SHOPS/4:
        SUPER_NEW = BIGFARMS*4
    else: 
        SUPER_NEW = SHOPS/4
    values = ((HOUSES, NEW_RES, RESIDENTIAL), (SHOPS, NEW_COMM, COMMERCIAL), (FARMS, NEW_FARM, FARM), (BIGFARMS, NEW_BIGFARM, FARM))
    for present, future, zone in values:
        diff = future - present
        if diff > 0 and cells[r][c].type == UNBUILT:
            print "have", present, "; need", future, "; missing:", diff
            for i in range(diff):
                dopat(block, cells, zone)
    if SUPERMARKETS < SUPER_NEW and BIGFARMS >= 1:
        diffs = SUPER_NEW - SUPERMARKETS 
        print "THERE SHOULD BE", SUPER_NEW, "NEW SUPERMARKETS"
        for i in range(diffs):
            #if cells[r][c].type == COMMERCIAL:
            dopat(block, cells, SUPERMARKET)

    ##### growing cities
    if HOUSES > 12 and HOUSES < 80:
        for i in range(HOUSES/12):
            if cells[r][c].type == UNBUILT:
                dopat(block, cells, RESIDENTIAL)

    ##### shrinking cities
    if HOUSES < 8 or HOUSES >= 84:
        for i in range(HOUSES/4):
            if cells[r][c].type == RESIDENTIAL:
                dopat(block, cells, UNBUILT)

    ##### growing bigfarms
    if BIGFARMS >=1:
        for i in range(BIGFARM):
            if cells[r-1][c-1].type == FARM and cells[r][c].type_next == UNBUILT:
                cells[r][c].type_next = FARM
    
    ##### supermarkets need big farms:
    if BIGFARMS*4 < SUPERMARKETS:
        for i in range(SUPERMARKETS%4):
            if cells[r][c].type == SUPERMARKET:
                dopat(block, cells, COMMERCIAL)    

    ##### end of the game        
    if SUPERMARKETS >= 12:
        print "SUPERMARKETS WON!"
        screen.blit(won, (200, 300, 400, 300))
        pygame.time.wait(2000)
        pygame.display.update()
        running = False
        begin()
                        

#------------------game functions-----------------------#
## IN BETWEEN
def calculation():
    (S, O) = read_db()
    SUM = S+O
    #print SUM
    super = float(S)/SUM
    open = float(O)/SUM
    INIT_RES = int(25*open)
    INIT_COMM = int(15*super)
    INIT_FARM = int(15*open)
    if INIT_FARM >= 4:
        INIT_BIGFARM = int(2*super)
    else:
        INIT_BIGFARM = 0
    return INIT_RES, INIT_COMM, INIT_FARM, INIT_BIGFARM

def seed_city():
    global ncells, nrows, ncols
    for row in range(nrows):
        for celija in range(ncols):
            cells[r][c].type_next = UNBUILT
            dopat(block, cells, UNBUILT)
    count() 
    (INIT_RES, INIT_COMM, INIT_FARM, INIT_BIGFARM) = calculation()
    print "reading the database...."
    print "THE PEOPLE'S VOTES CREATE A CITY WITH"
    print INIT_RES, "HOUSES,", INIT_COMM, "SHOPS,", INIT_FARM, "FARMS, WHICH ARE OPTIONALLY COMBINED TO ENGAGE IN", INIT_BIGFARM, "INDUSTRIAL FARM"
    for i in range(int(INIT_RES/3)):
        dopat(residential, cells, RESIDENTIAL)
    if INIT_RES/3 != 0:
        for i in range(INIT_RES%3):
            if cells[r][c].type == UNBUILT:
                dopat(block, cells, RESIDENTIAL)
    if INIT_BIGFARM != 0 and INIT_FARM >= 4:
        for i in range(INIT_BIGFARM):
             if cells[r][c].type == UNBUILT:
                dopat(large_farm, cells, FARM)
    FARMS = INIT_FARM - INIT_BIGFARM*4
    for i in range(FARMS):
            if cells[r][c].type == UNBUILT:
                dopat(block, cells, FARM)

#    FARMS = INIT_FARM - INIT_BIGFARM*4
#    SHOPS = INIT_COMM - INIT_FARM/3           
#        
#    zones = [(SHOPS, COMMERCIAL), (FARMS, FARM)]
#    for value, zone in zones:
#        for i in range(value):
#            if Cell(r, c).type == UNBUILT:
#                dopat(block, cells, zone)

def begin():
    global cells, draw_type, running
    draw_type = UNBUILT
    for c in range(ncols):
        for r in range(nrows):
            cells[r][c].draw(screen)
    seed_city()
        
    introduction = [welcome, intro, run]
        
    for i in range(len(introduction)):
        screen.blit(bg_stop, (0, 0))
        for c in range(ncols):
            for r in range(nrows):
                cells[r][c].draw(screen)
        screen.blit(introduction[i], (148, 282, screen_width, screen_height))
        pygame.display.update()
        pygame.time.wait(2000)


#------------------pygame init--------------------------#
pygame.init()

screen = pygame.display.set_mode((704, 704), 0, 16)

bg = pygame.Surface(screen.get_size())
bg = bg.convert()
bg.fill((73, 86, 16))

bg_stop = pygame.Surface(screen.get_size())
bg_stop = bg_stop.convert()
bg_stop.fill((73, 86, 16))

cells = []
(screen_width, screen_height) = screen.get_size()
ncols = screen_width / CELL_WIDTH
nrows = screen_height / CELL_HEIGHT
ncells = ncols * nrows

running = False
drawing = False
draw_type = UNBUILT

# init the cells
for r in range(nrows):
    currow = []
    cells.append(currow)
    for c in range(ncols):
        cell = Cell(r, c)
        currow.append(cell)


#---------------------main loop------------------------#
begin()

while True:
    
    pygame.display.update()

    which_key()
        
    if running:
        screen.blit(bg, (0, 0))
        count()
        run_game()

    else:
        screen.blit(bg_stop, (0, 0))

    for c in range(ncols):
        for r in range(nrows):
            cells[r][c].draw(screen)

    pygame.time.wait(550)


# Copyright 2010 Selena Savic
# released under GNU GPL
"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
